﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  [View("WindowedSymbol View")]
  [Content(typeof(WindowedSymbol), true)]
  public partial class WindowedSymbolView : SymbolView {
    public new WindowedSymbol Content {
      get { return (WindowedSymbol)base.Content; }
      set { base.Content = value; }
    }

    public WindowedSymbolView() {
      InitializeComponent();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.Changed += new EventHandler(Content_Changed);
    }

    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
      Content.Changed -= new EventHandler(Content_Changed);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      UpdateControl();
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();

      initializationGroupBox.Enabled = Content != null && Content.EnableWindowing;
      mutationGroupBox.Enabled = Content != null && Content.EnableWindowing;

      initOffsetMuTextBox.ReadOnly = ReadOnly;
      initOffsetSigTextBox.ReadOnly = ReadOnly;
      initLengthMuTextBox.ReadOnly = ReadOnly;
      initLengthSigTextBox.ReadOnly = ReadOnly;
      mutOffsetMuTextBox.ReadOnly = ReadOnly;
      mutOffsetSigTextBox.ReadOnly = ReadOnly;
      mutLengthMuTextBox.ReadOnly = ReadOnly;
      mutLengthSigTextBox.ReadOnly = ReadOnly;
    }

    #region content event handlers
    private void Content_Changed(object sender, EventArgs e) {
      UpdateControl();
    }
    #endregion

    #region control event handlers
    private void enableWindowedCheckBox_CheckedChanged(object sender, EventArgs e) {
      if (Content == null) return;
      Content.EnableWindowing = enableWindowedCheckBox.Checked;
    }

    private void initOffsetMuTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(initOffsetMuTextBox.Text, out double initOffsetMu)) {
        Content.OffsetMu = initOffsetMu;
        errorProvider.SetError(initOffsetMuTextBox, string.Empty);
      } else {
        errorProvider.SetError(initOffsetMuTextBox, "Invalid value");
      }
    }
    private void initOffsetSigTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(initOffsetSigTextBox.Text, out double initOffsetSig)) {
        Content.OffsetSigma = initOffsetSig;
        errorProvider.SetError(initOffsetSigTextBox, string.Empty);
      } else {
        errorProvider.SetError(initOffsetSigTextBox, "Invalid value");
      }
    }
    private void initLengthMuTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(initLengthMuTextBox.Text, out double initLengthMu)) {
        Content.LengthMu = initLengthMu;
        errorProvider.SetError(initLengthMuTextBox, string.Empty);
      } else {
        errorProvider.SetError(initLengthMuTextBox, "Invalid value");
      }
    }
    private void initLengthSigTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(initLengthSigTextBox.Text, out double initLengthSig)) {
        Content.LengthSigma = initLengthSig;
        errorProvider.SetError(initLengthSigTextBox, string.Empty);
      } else {
        errorProvider.SetError(initLengthSigTextBox, "Invalid value");
      }
    }

    private void mutOffsetMuTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(mutOffsetMuTextBox.Text, out double mutOffsetMu)) {
        Content.ManipulatorOffsetMu = mutOffsetMu;
        errorProvider.SetError(mutOffsetMuTextBox, string.Empty);
      } else {
        errorProvider.SetError(mutOffsetMuTextBox, "Invalid value");
      }
    }
    private void mutOffsetSigTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(mutOffsetSigTextBox.Text, out double mutOffsetSig)) {
        Content.ManipulatorOffsetSigma = mutOffsetSig;
        errorProvider.SetError(mutOffsetSigTextBox, string.Empty);
      } else {
        errorProvider.SetError(mutOffsetSigTextBox, "Invalid value");
      }
    }
    private void mutLengthMuTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(mutLengthMuTextBox.Text, out double mutLengthMu)) {
        Content.ManipulatorLengthMu = mutLengthMu;
        errorProvider.SetError(mutLengthMuTextBox, string.Empty);
      } else {
        errorProvider.SetError(mutLengthMuTextBox, "Invalid value");
      }
    }
    private void mutLengthSigTextBox_TextChanged(object sender, EventArgs e) {
      if (Content == null) return;
      if (double.TryParse(mutLengthSigTextBox.Text, out double mutLengthSig)) {
        Content.ManipulatorLengthSigma = mutLengthSig;
        errorProvider.SetError(mutLengthSigTextBox, string.Empty);
      } else {
        errorProvider.SetError(mutLengthSigTextBox, "Invalid value");
      }
    }
    #endregion

    #region helpers
    private void UpdateControl() {
      enableWindowedCheckBox.Checked = Content?.EnableWindowing ?? false;

      initOffsetMuTextBox.Text = Content?.OffsetMu.ToString() ?? string.Empty;
      initOffsetSigTextBox.Text = Content?.OffsetSigma.ToString() ?? string.Empty;
      initLengthMuTextBox.Text = Content?.LengthMu.ToString() ?? string.Empty;
      initLengthSigTextBox.Text = Content?.LengthSigma.ToString() ?? string.Empty;

      mutOffsetMuTextBox.Text = Content?.ManipulatorOffsetMu.ToString() ?? string.Empty;
      mutOffsetSigTextBox.Text = Content?.ManipulatorOffsetSigma.ToString() ?? string.Empty;
      mutLengthMuTextBox.Text = Content?.ManipulatorLengthMu.ToString() ?? string.Empty;
      mutLengthSigTextBox.Text = Content?.ManipulatorLengthSigma.ToString() ?? string.Empty;
    
      SetEnabledStateOfControls();
    }
    #endregion
  }
}
